import { NonEmptyArray } from 'fp-ts/lib/NonEmptyArray';
import * as t from 'io-ts';
import * as Address_2d0a_ from './components/address';
import * as Common_ffba_ from './components/common';
import * as Fare_a3ab_ from './components/fare';
import * as I18n_b70d_ from './components/i18n';
import * as Units_c404_ from './components/units';
import * as PersonalDocument_9750_ from './personal-document';
export type Defined = {} | null;
export declare class DefinedType extends t.Type<Defined> {
    readonly _tag: 'DefinedType';
    constructor();
}
export type DefinedC = {} & DefinedType;
export declare const Defined: DefinedC;
export type NullBrand = {
    readonly Null: unique symbol;
};
export type NullC = t.BrandC<t.UnknownC, NullBrand>;
export declare const Null: NullC;
export type Null = t.TypeOf<typeof Null>;
export declare const schemaId = "https://schemas.maas.global/core/customer.json";
export type Customer = t.Branded<{
    identityId?: Units_c404_.IdentityId;
    honorifics?: string;
    firstName?: Common_ffba_.PersonalName;
    lastName?: Common_ffba_.PersonalName;
    firstNameLocalized?: Common_ffba_.PersonalName;
    lastNameLocalized?: Common_ffba_.PersonalName;
    sex?: string;
    phone?: Common_ffba_.Phone;
    email?: Common_ffba_.Email;
    address?: Address_2d0a_.Address;
    city?: Address_2d0a_.City;
    state?: Address_2d0a_.State;
    country?: Address_2d0a_.Country;
    zipCode?: Address_2d0a_.ZipCode;
    locale?: I18n_b70d_.Locale;
    appInstanceId?: Common_ffba_.AppInstanceId;
    opaqueId?: Common_ffba_.OpaqueId;
    opaqueIdShort?: Common_ffba_.OpaqueIdShort;
    clientId?: Common_ffba_.ClientId;
    dob?: boolean | Units_c404_.IsoDate;
    ssid?: boolean | Common_ffba_.Ssid;
    'ssid-fi'?: boolean | Common_ffba_.SsidFI;
    documents?: Array<PersonalDocument_9750_.PersonalDocument>;
    balances?: Record<string, (({
        currency?: Units_c404_.Currency | Common_ffba_.MetaCurrencyWMP;
        amount?: number;
    } & Record<string, unknown>) & {
        currency: Defined;
        amount: Defined;
    }) | (({
        currency?: Common_ffba_.MetaCurrencyTOKEN | Fare_a3ab_.TokenId;
        tokenId?: Fare_a3ab_.TokenId;
        amount?: number | Null;
    } & Record<string, unknown>) & {
        currency: Defined;
        tokenId: Defined;
        amount: Defined;
    })>;
    referral?: {
        code?: string;
    } & Record<string, unknown>;
    subscriberType?: string;
    authToken?: Common_ffba_.EncodedQueryParam;
    cugHome?: string;
}, CustomerBrand>;
export type CustomerC = t.BrandC<t.PartialC<{
    identityId: typeof Units_c404_.IdentityId;
    honorifics: t.StringC;
    firstName: typeof Common_ffba_.PersonalName;
    lastName: typeof Common_ffba_.PersonalName;
    firstNameLocalized: typeof Common_ffba_.PersonalName;
    lastNameLocalized: typeof Common_ffba_.PersonalName;
    sex: t.StringC;
    phone: typeof Common_ffba_.Phone;
    email: typeof Common_ffba_.Email;
    address: typeof Address_2d0a_.Address;
    city: typeof Address_2d0a_.City;
    state: typeof Address_2d0a_.State;
    country: typeof Address_2d0a_.Country;
    zipCode: typeof Address_2d0a_.ZipCode;
    locale: typeof I18n_b70d_.Locale;
    appInstanceId: typeof Common_ffba_.AppInstanceId;
    opaqueId: typeof Common_ffba_.OpaqueId;
    opaqueIdShort: typeof Common_ffba_.OpaqueIdShort;
    clientId: typeof Common_ffba_.ClientId;
    dob: t.UnionC<[t.BooleanC, typeof Units_c404_.IsoDate]>;
    ssid: t.UnionC<[t.BooleanC, typeof Common_ffba_.Ssid]>;
    'ssid-fi': t.UnionC<[t.BooleanC, typeof Common_ffba_.SsidFI]>;
    documents: t.ArrayC<typeof PersonalDocument_9750_.PersonalDocument>;
    balances: t.RecordC<t.StringC, t.UnionC<[
        t.IntersectionC<[
            t.IntersectionC<[
                t.PartialC<{
                    currency: t.UnionC<[
                        typeof Units_c404_.Currency,
                        typeof Common_ffba_.MetaCurrencyWMP
                    ]>;
                    amount: t.NumberC;
                }>,
                t.RecordC<t.StringC, t.UnknownC>
            ]>,
            t.TypeC<{
                currency: typeof Defined;
                amount: typeof Defined;
            }>
        ]>,
        t.IntersectionC<[
            t.IntersectionC<[
                t.PartialC<{
                    currency: t.UnionC<[
                        typeof Common_ffba_.MetaCurrencyTOKEN,
                        typeof Fare_a3ab_.TokenId
                    ]>;
                    tokenId: typeof Fare_a3ab_.TokenId;
                    amount: t.UnionC<[t.NumberC, typeof Null]>;
                }>,
                t.RecordC<t.StringC, t.UnknownC>
            ]>,
            t.TypeC<{
                currency: typeof Defined;
                tokenId: typeof Defined;
                amount: typeof Defined;
            }>
        ]>
    ]>>;
    referral: t.IntersectionC<[
        t.PartialC<{
            code: t.StringC;
        }>,
        t.RecordC<t.StringC, t.UnknownC>
    ]>;
    subscriberType: t.StringC;
    authToken: typeof Common_ffba_.EncodedQueryParam;
    cugHome: t.StringC;
}>, CustomerBrand>;
export declare const Customer: CustomerC;
export type CustomerBrand = {
    readonly Customer: unique symbol;
};
/** require('io-ts-validator').validator(nonEmptyArray(Customer)).decodeSync(examplesCustomer) // => examplesCustomer */
export declare const examplesCustomer: NonEmptyArray<Customer>;
export default Customer;
import "jest";
